#!/bin/bash
#
# Wrapper script for cross-reference counting program to extract the
# block size, etc. of a file system.  Can be used on a mounted file
# system if you specify the current mountpoint.
#
# (C) 2006-2007 Karuna Sagar K <karunasagark@gmail.com>
# (C) 2007 Val Henson <val@nmt.edu>
#
# Licensed under GPLv2
#

if [ $# -lt 3 ]
then
	echo "Usage: $0 <image name> <super block dump file> <mount point> [bg size in MB]"
	exit 1
fi

make

imgname=$1
dmpfile=$2
mntpt=$3
if [ -n "$4" ]; then
    desired_group_size=$4
    desired_group_size=$((desired_group_size * 1024 * 1024))
fi

dumpe2fs -h $imgname > $dmpfile 2>/dev/null

blk_per_grp=`grep 'Blocks per group' $dmpfile | cut -d : -f 2`
ind_per_grp=`grep 'Inodes per group' $dmpfile | cut -d : -f 2`
blk_sz=`grep 'Block size' $dmpfile | cut -d : -f 2`
blk_count=`grep 'Block count' $dmpfile | cut -d : -f 2`
free_blk_count=`grep 'Free blocks' $dmpfile | cut -d : -f 2`
blk_grp=`expr $blk_count \/ $blk_per_grp`
if [ `expr $blk_count \% $blk_per_grp` -ne 0 ]
then
	blk_grp=`expr $blk_grp + 1`
fi

# Scale blocks per group and inodes per group to desired block group size

multiplier=1
if [ -n "$desired_group_size" ]; then
    multiplier=$((desired_group_size / (blk_sz * blk_per_grp)))
fi

blk_per_grp=$((blk_per_grp * multiplier))
ind_per_grp=$((ind_per_grp * multiplier))

# XXX Check for already mounted
mount -t ext2 -o loop -o ro $imgname $mntpt > /dev/null
./cref -m $mntpt -b $blk_per_grp -i $ind_per_grp -s $blk_sz -g $blk_grp -n $blk_count -f $free_blk_count
# XXX Check if we had to mount it
umount -f $mntpt > /dev/null
